/*
 *
 * $Source: /filesv/usr/local/proj/sphinx/spx2/src/lib/cfile/RCS/in_clm_ctx.c,v $
 *
 *
 *  MODULE NAME:    in_clm_ctx.c
 *
 *
 *  AUTHORS:
 *
 *	K. Alagappan
 *
 */


/*
 * COPYRIGHT (C) 1992 DIGITAL EQUIPMENT CORPORATION
 * ALL RIGHTS RESERVED
 *
 * "Digital Equipment Corporation authorizes the reproduction,
 * distribution and modification of this software subject to the following
 * restrictions:
 * 
 * 1.  Any partial or whole copy of this software, or any modification
 * thereof, must include this copyright notice in its entirety.
 *
 * 2.  This software is supplied "as is" with no warranty of any kind,
 * expressed or implied, for any purpose, including any warranty of fitness 
 * or merchantibility.  DIGITAL assumes no responsibility for the use or
 * reliability of this software, nor promises to provide any form of 
 * support for it on any basis.
 *
 * 3.  Distribution of this software is authorized only if no profit or
 * remuneration of any kind is received in exchange for such distribution. 
 * 
 * 4.  This software and all application programs are to be used only for
 * non-commercial purposes. However, media costs associated with the
 * distribution of the software or application programs may be recovered.
 *
 */


#include <stdio.h>
#include <cdc.h>
#include <cdc_db.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>

extern int cdc_debug;

/*
 * in_ctx() is used to initialize the context store.  It creates the
 * file to contain the tickets and writes the given user's name "pname"
 * in the file.  in_tkt() returns ASUCCESS on success, or AFAILURE if
 * something goes wrong.
 */

in_clm_ctx(file)
    char *file;
{
    int tktfile, creat();
    short me, getuid();
    struct stat buf;
    int fd;
    register int i;
    char charbuf[BUFSIZ];

    me = getuid ();
    if (lstat(file,&buf) == 0) {
	if (buf.st_uid != me || !(buf.st_mode & S_IFREG) ||
	    buf.st_mode & 077) {
	    if (cdc_debug)
		fprintf(stderr,"Error initializing %s",file);
	    return(AFAILURE);
	}
	/* file already exists, and permissions appear ok, so nuke it */
	if ((fd = open(file, O_RDWR, 0)) < 0)
	    goto out; /* can't zero it, but we can still try truncating it */

	bzero(charbuf, sizeof(charbuf));

	for (i = 0; i < buf.st_size; i += sizeof(charbuf))
	    if (write(fd, charbuf, sizeof(charbuf)) != sizeof(charbuf)) {
		(void) fsync(fd);
		(void) close(fd);
		goto out;
	    }
	
	(void) fsync(fd);
	(void) close(fd);
    }
 out:
    if ((tktfile = creat(file,0600)) < 0) {
	if (cdc_debug)
	    fprintf(stderr,"Error creating %s\n",file);
        return(AFAILURE);
    }
    if (lstat(file,&buf) < 0) {
	if (cdc_debug)
	    fprintf(stderr,"Error getting lstat on %s\n",file);
        return(AFAILURE);
    }

    if (buf.st_uid != me || !(buf.st_mode & S_IFREG) ||
        buf.st_mode & 077) {
	if (cdc_debug)
	    fprintf(stderr,"Error protection mode on %s\n",file);
        return(AFAILURE);
    }
    (void) close(tktfile);
    return(ASUCCESS);
}
